-- rand_old.e - PRNG ('clean' version) - HIP 2.0
-- Copyright (C) 2000  Davi Tassinari de Figueiredo
--
-- This program is distributed under the terms of the GNU General
-- Public License. Please read the documentation for more information.
--
-- This file works the same as rand.e, but it does not use any
-- machine code routines, so it should be easier to understand and port.
--
-- The code in this file is based on arcf_old.e.

atom a,b
sequence state

a=1 b=1 state={}

procedure arcfour_init_encrypt(sequence key)
    -- Initialize the state array
    atom temp

    state=repeat(0,256)
    for pos=1 to length(state) do
	state[pos]=pos-1
    end for

    a=1     b=1

    for n=1 to 256 do
	b=and_bits(b+key[a]+state[n],#FF)
	if b=0 then b=256 end if

	-- Swap state[n] and state[b]
	temp=state[n]
	state[n]=state[b]
	state[b]=temp
	a=remainder(a,length(key))+1
    end for

    a=1     b=1

end procedure


constant byte_vals = {#1, #100, #10000, #1000000}

function arcfour_32bit_number ()
    atom index, number

    number = 0

    for byte=1 to 4 do
	a=and_bits(a,#FF)+1
	b=and_bits(b+state[a],#FF)
	if b=0 then b=256 end if

	-- Swap state[a] and state[b]
	index=state[a]
	state[a]=state[b]
	state[b]=index

	number = number +
	 state[and_bits(state[a]+state[b],#FF)+1] * byte_vals [byte]
    end for

    return number
end function

atom divisor

global function random ()
    return remainder (arcfour_32bit_number(), divisor) + 1
end function

global procedure init_random (sequence key, atom max)
    atom junk

    -- Call initialization routine
    arcfour_init_encrypt (key)

    -- Set divisor
    divisor = max

    -- Discard the first 256 numbers generated, further initializing the generator
    for n = 1 to 256 do
	junk = random()
    end for


end procedure


